{% extends 'base/login' %}
{% block head %}
{% endblock %}
{% block body %}
<div class='d-flex flex-row justify-content-center'>
    <div style='width: 320px' class='mt-5'>
{% block content %}
{% endblock %}
    </div>
</div>
{% endblock %}
