{% extends 'base.html' %}
{% block header %}
    {% include 'header.html' %}
{% endblock %}
{% block content %}
{% if error == None %}
{% set post = request.post() %}
{% else %}
{% set post = {} %}
{% endif %}
<div id='content' class='container'>
    <nav class='nav nav-tabs mt-3'>
        <a class='nav-link' href='{{ url('/users/' + user.screen_name + '/settings/profile') }}'>プロフィール</a>
        <a class='nav-link active' href='{{ url('/users/' + user.screen_name + '/settings/password') }}'>パスワード</a>
        <a class='nav-link' href='{{ url('/users/' + user.screen_name + '/settings/notification') }}'>通知</a>
    </nav>
    <h2 class='mt-4 mb-3'>パスワード</h2>
    <form class='mb-4' method='POST'>
        <div class='mb-3'>
            <label>新しいパスワード</label>
            <input type='password' name='password' class='form-control' />
        </div>
        <div class='mb-3'>
            <label>新しいパスワード(確認)</label>
            <input type='password' name='password_' class='form-control' />
        </div>
        {{ forms.csrf(request) }}
        <p>{{ error }}</p>
        <button class='btn btn-primary'>更新</button>
    </form>
</div>
{% endblock %}
