{% extends 'login_base.html' %}
{% block content %}
<form method='post' action='' class='entrance'>
    <div class='main mb-4'>
        <h1 class='mb-4'>ログイン</h1>
        <div class='mb-2'>
            <input type='text' name='screen_name' placeholder='ログイン名' value='{{ screen_name }}' class='form-control'/>
        </div>
        <div>
            <input type='password' name='password' placeholder='パスワード' value='{{ password }}' class='form-control'/>
        </div>
    </div>
    {{ forms.csrf(request) }}
    <div class='d-grid'>
        <button type="submit" class='btn btn-primary'>ログイン</button>
    </div>
    <p class='error'>{{ error }}</p>
</form>
{% endblock %}
