header {
  height: 56px;
  display: grid;
  grid-template-columns: 1fr auto;
  background: var(--header-background);
  position: sticky;
  top: 0;
  color: var(--header-text);
}
header #logo {
  height: 40px;
  display: block;
}
header #logo_container {
  padding: 8px 16px;
}
header #user-menu #menu-status {
  display: none;
}
header #user-menu label {
  padding: 8px 16px;
  display: grid;
  grid-gap: 8px;
  grid-template-columns: 32px 1fr;
  align-items: center;
  height: 40px;
  font-weight: bold;
  background: #0001;
  width: 160px;
}
header #user-menu .user-icon {
  width: 32px;
  height: 32px;
}
header menu {
  display: flex;
  opacity: 0;
  flex-direction: column;
  justify-content: flex-end;
  position: absolute;
  top: 40px;
  pointer-events: none;
  width: 200px;
  right: 0;
  display: block;
  margin: 0;
  padding: 16px;
  transition: all 0.2s;
  background: var(--main-background);
  box-shadow: 0 0 10px #0004;
}
header menu > div {
  border-bottom: 1px solid var(--main-decoration);
  padding-bottom: 8px;
  margin-bottom: 8px;
}
header menu div:last-child {
  border: none;
  padding-bottom: 0;
  margin-bottom: 0;
}
header menu a {
  display: grid;
  grid-template-columns: 32px 1fr;
  grid-gap: 8px;
  width: 100%;
  height: 40px;
  justify-content: center;
  align-items: center;
  text-decoration: none;
  padding: 0 4px;
  font-weight: bold;
  color: var(--main-text);
  transition: all 0.2s;
}
header menu a svg {
  width: 32px;
  height: 32px;
  fill: var(--main-text);
  transition: all 0.2s;
}
header menu a:hover {
  color: var(--main-color);
}
header menu a:hover svg {
  fill: var(--main-color);
}
header #menu-status:checked ~ menu {
  opacity: 1;
  pointer-events: all;
  top: 64px;
}

.user-icon {
  border-radius: 50%;
}

.icon {
  object-fit: cover;
}

.item .icon {
  border-radius: 50%;
}

.no-item {
  color: var(--main-text-lighter);
  font-size: 0.8rem;
  display: flex;
  justify-content: center;
  align-items: center;
  padding: 24px 0;
}

.member-append-list .item-container {
  display: grid;
  grid-template-columns: 24px 1fr;
}
.member-append-list .item-container input[type=checkbox] {
  margin: 30px 6px;
  width: 12px;
  height: 12px;
}
.member-append-list .item {
  display: grid;
  grid-template-columns: 56px 1fr;
  grid-gap: 16px;
  text-decoration: none;
  color: var(--main-text);
  padding: 8px;
}
.member-append-list .item .user-icon, .member-append-list .item .icon {
  display: block;
  width: 56px;
  height: 56px;
  box-sizing: border-box;
  border: 2px solid #0001;
}
.member-append-list .item .user-icon {
  border-radius: 28px;
}
.member-append-list .item .icon {
  border-radius: 4px;
}
.member-append-list .item .detail {
  display: flex;
  justify-content: space-around;
  flex-direction: column;
  padding: 8px 0;
}
.member-append-list .item .detail .name {
  font-size: 1rem;
  font-weight: bold;
}
.member-append-list .item .detail .screen-name {
  font-size: 0.85rem;
  color: var(--main-text-lighter);
}

.group-list .group-item {
  display: grid;
  grid-template-columns: auto 1fr;
}
.group-list .group-item .branch {
  grid-column: 1/2;
  width: 0px;
}
.group-list .group-item .profile {
  display: grid;
  text-decoration: none;
  color: var(--main-text);
  grid-column: 2/3;
  grid-template-columns: 56px 1fr;
  grid-gap: 16px;
  padding: 8px;
}
.group-list .group-item .profile .icon {
  width: 56px;
  height: 56px;
  border-radius: 4px;
  border: 2px solid #0001;
  box-sizing: border-box;
}
.group-list .group-item .profile .detail {
  display: flex;
  justify-content: space-around;
  flex-direction: column;
  padding: 8px 0;
}
.group-list .group-item .profile .detail .name {
  font-size: 1rem;
  font-weight: bold;
}
.group-list .group-item .profile .detail .screen-name {
  font-size: 0.85rem;
  color: var(--main-text-lighter);
}
.group-list .children {
  grid-column: 2/3;
  position: relative;
}
.group-list .children::before {
  content: "";
  display: block;
  position: absolute;
  width: 2px;
  height: calc(100% - 45px);
  left: 16px;
  background: var(--main-color);
}
.group-list .children .branch {
  width: 24px;
  display: block;
  position: relative;
}
.group-list .children .branch::before {
  content: "";
  display: block;
  position: absolute;
  box-sizing: border-box;
  border-left: 2px solid var(--main-color);
  border-bottom: 2px solid var(--main-color);
  width: 8px;
  height: 40px;
  left: 16px;
  top: -8px;
  border-bottom-left-radius: 4px;
}

.user-list h4,
.invite-list h4 {
  margin: 16px 0 8px;
  font-size: 0.75rem;
}
.user-list .user-item,
.user-list .invite-item,
.invite-list .user-item,
.invite-list .invite-item {
  display: grid;
  grid-template-columns: 56px 1fr;
  grid-gap: 16px;
  text-decoration: none;
  color: var(--main-text);
  padding: 8px;
}
.user-list .user-item .user-icon, .user-list .user-item .icon,
.user-list .invite-item .user-icon,
.user-list .invite-item .icon,
.invite-list .user-item .user-icon,
.invite-list .user-item .icon,
.invite-list .invite-item .user-icon,
.invite-list .invite-item .icon {
  display: block;
  width: 56px;
  height: 56px;
  box-sizing: border-box;
  border: 2px solid #0001;
}
.user-list .user-item .user-icon,
.user-list .invite-item .user-icon,
.invite-list .user-item .user-icon,
.invite-list .invite-item .user-icon {
  border-radius: 28px;
}
.user-list .user-item .icon,
.user-list .invite-item .icon,
.invite-list .user-item .icon,
.invite-list .invite-item .icon {
  border-radius: 4px;
}
.user-list .user-item .detail,
.user-list .invite-item .detail,
.invite-list .user-item .detail,
.invite-list .invite-item .detail {
  display: flex;
  justify-content: space-around;
  flex-direction: column;
  padding: 8px 0;
}
.user-list .user-item .detail .name,
.user-list .invite-item .detail .name,
.invite-list .user-item .detail .name,
.invite-list .invite-item .detail .name {
  font-size: 1rem;
  font-weight: bold;
}
.user-list .user-item .detail .screen-name,
.user-list .invite-item .detail .screen-name,
.invite-list .user-item .detail .screen-name,
.invite-list .invite-item .detail .screen-name {
  font-size: 0.85rem;
  color: var(--main-text-lighter);
}
.user-list .user-item .detail .buttons,
.user-list .invite-item .detail .buttons,
.invite-list .user-item .detail .buttons,
.invite-list .invite-item .detail .buttons {
  text-align: right;
  font-size: 0.8rem;
}
.user-list .user-item .detail .buttons a,
.user-list .invite-item .detail .buttons a,
.invite-list .user-item .detail .buttons a,
.invite-list .invite-item .detail .buttons a {
  display: inline-block;
}
.user-list .user-item .detail .buttons input,
.user-list .invite-item .detail .buttons input,
.invite-list .user-item .detail .buttons input,
.invite-list .invite-item .detail .buttons input {
  display: block;
  position: absolute;
  overflow: hidden;
  opacity: 0;
}

.member-control-list h4 {
  margin: 16px 0 8px;
  font-size: 0.75rem;
}
.member-control-list .member-control-item {
  display: grid;
  grid-template-columns: 56px 1fr auto;
  grid-gap: 16px;
  text-decoration: none;
  color: var(--main-text);
  padding: 8px;
}
.member-control-list .member-control-item .user-icon, .member-control-list .member-control-item .icon {
  display: block;
  width: 56px;
  height: 56px;
  box-sizing: border-box;
  border: 2px solid #0001;
}
.member-control-list .member-control-item .user-icon {
  border-radius: 28px;
}
.member-control-list .member-control-item .icon {
  border-radius: 4px;
}
.member-control-list .member-control-item .detail {
  display: flex;
  justify-content: space-around;
  flex-direction: column;
  padding: 8px 0;
}
.member-control-list .member-control-item .detail .name {
  font-size: 1rem;
  font-weight: bold;
}
.member-control-list .member-control-item .detail .screen-name {
  font-size: 0.85rem;
  color: var(--main-text-lighter);
}
.member-control-list .member-control-item .actions {
  display: flex;
  justify-content: center;
  align-items: center;
}
.member-control-list .member-control-item .actions a {
  margin-left: 4px;
}

.group-list .group-item {
  display: grid;
  grid-template-columns: auto 1fr;
}
.group-list .group-item .branch {
  grid-column: 1/2;
  width: 0px;
}
.group-list .group-item .profile {
  display: grid;
  text-decoration: none;
  color: var(--main-text);
  grid-column: 2/3;
  grid-template-columns: 56px 1fr;
  grid-gap: 16px;
  padding: 8px;
}
.group-list .group-item .profile .icon {
  width: 56px;
  height: 56px;
  border-radius: 4px;
  border: 2px solid #0001;
  box-sizing: border-box;
}
.group-list .group-item .profile .detail {
  display: flex;
  justify-content: space-around;
  flex-direction: column;
  padding: 8px 0;
}
.group-list .group-item .profile .detail .name {
  font-size: 1rem;
  font-weight: bold;
}
.group-list .group-item .profile .detail .screen-name {
  font-size: 0.85rem;
  color: var(--main-text-lighter);
}
.group-list .children {
  grid-column: 2/3;
}
.group-list .children .branch {
  width: 24px;
  display: block;
  position: relative;
}
.group-list .children .branch::before {
  content: "";
  display: block;
  position: absolute;
  box-sizing: border-box;
  border-left: 2px solid var(--main-color);
  border-bottom: 2px solid var(--main-color);
  width: 8px;
  height: 40px;
  left: 16px;
  top: -8px;
  border-bottom-left-radius: 4px;
}
.group-list .children .item + .item > .branch::after {
  content: "";
  display: block;
  position: absolute;
  box-sizing: border-box;
  border-left: 2px solid var(--main-color);
  width: 8px;
  height: 38px;
  left: 16px;
  top: -38px;
}

.app-list {
  display: grid;
  grid-template-columns: repeat(auto-fill, minmax(280px, 1fr));
  justify-content: start;
}
.app-list .app-item {
  display: grid;
  text-decoration: none;
  color: var(--main-text);
  grid-template-columns: 56px 1fr;
  grid-gap: 16px;
  padding: 8px;
}
.app-list .app-item .icon {
  width: 56px;
  height: 56px;
  border-radius: 4px;
  border: 2px solid #0001;
  box-sizing: border-box;
}
.app-list .app-item .detail {
  display: flex;
  flex-direction: column;
  padding: 8px 0;
}
.app-list .app-item .detail .name {
  font-size: 1rem;
  font-weight: bold;
}
.app-list .app-item .detail .screen-name {
  font-size: 0.85rem;
  color: var(--main-text-lighter);
}

/*# sourceMappingURL=style.css.map */
